#Function to simulate virus transmission in a migratory population (baseline model)
migdelayStoSIIRR<-function(yearday, year, tm_step, S0, I0, N, r, beta, u, uh, ud, b, m, Np, v, realdata=FALSE){
  # Input: yearday: days of a year; year: years; tm_step: time step;
  #         S0, I0: initial states matrix; N: population size
  #         r: infection recovery rate, day; beta: contact transmission matrix;
  #         u: natural mortality rate; uh: hunting mortality rate; b: birth rate matrix;
  #         m: migration rate matrix
  #         NP: patch number
  #         v: migration recovery rate
  #
  # output: S, I, R for all time steps
  
  tm_vec=seq(1,year*yearday+1,by=tm_step)
  tm_sz=length(tm_vec)+1;
  S=I1=I2=R1=R2=matrix(0,Np,tm_sz)
  
  S[,1]=round(S0,0); I1[,1]=round(I0,0); R1[,1]=round((N-S0-I0),0)
  
  #Temperal components
  Einf=vector(mode = "double",length = Np)
  Ebac=vector(mode = "double",length = Np)
  Erec1=vector(mode = "double",length = Np)
  Erec2=vector(mode = "double",length = Np)
  EmorS=vector(mode = "double",length = Np)
  EmorI1=vector(mode = "double",length = Np)
  EmorI2=vector(mode = "double",length = Np)
  EmorR1=vector(mode = "double",length = Np)
  EmorR2=vector(mode = "double",length = Np)
  Emord=vector(mode = "double",length = Np)
  EmigSout=vector(mode = "double",length = Np)
  EmigI2out=vector(mode = "double",length = Np)
  EmigR2out=vector(mode = "double",length = Np)
  EmigabI=vector(mode = "double",length = Np)
  EmigabR=vector(mode = "double",length = Np)
  Ebirth=vector(mode = "double",length = Np)
  
  stEinf=vector(mode = "double",length = Np)
  stEbac=vector(mode = "double",length = Np)
  stErec1=vector(mode = "double",length = Np)
  stErec2=vector(mode = "double",length = Np)
  stEmorS=vector(mode = "double",length = Np)
  stEmorI1=vector(mode = "double",length = Np)
  stEmorI2=vector(mode = "double",length = Np)
  stEmorR1=vector(mode = "double",length = Np)
  stEmorR2=vector(mode = "double",length = Np)
  stEmord=vector(mode = "double",length = Np)
  stEmigSout=vector(mode = "double",length = Np)
  stEmigI2out=vector(mode = "double",length = Np)
  stEmigR2out=vector(mode = "double",length = Np)
  stEmigabI=vector(mode = "double",length = Np)
  stEmigabR=vector(mode = "double",length = Np)
  stEbirth=vector(mode = "double",length = Np)
  
  sk1=vector(mode = "double",length = Np)
  i1k1=vector(mode = "double",length = Np)
  i2k1=vector(mode = "double",length = Np)
  r1k1=vector(mode = "double",length = Np)
  r2k1=vector(mode = "double",length = Np)
  ik1a=vector(mode = "double",length = Np)
  nk1=vector(mode = "double",length = Np)
  Ts1=vector(mode = "double",length = Np)
  Ti11=vector(mode = "double",length = Np)
  Ti21=vector(mode = "double",length = Np)
  Tr11=vector(mode = "double",length = Np)
  Tr21=vector(mode = "double",length = Np)
  
  sk2=vector(mode = "double",length = Np)
  i1k2=vector(mode = "double",length = Np)
  i2k2=vector(mode = "double",length = Np)
  r1k2=vector(mode = "double",length = Np)
  r2k2=vector(mode = "double",length = Np)
  ik2a=vector(mode = "double",length = Np)
  nk2=vector(mode = "double",length = Np)
  Ts2=vector(mode = "double",length = Np)
  Ti12=vector(mode = "double",length = Np)
  Ti22=vector(mode = "double",length = Np)
  Tr12=vector(mode = "double",length = Np)
  Tr22=vector(mode = "double",length = Np)
  
  sk3=vector(mode = "double",length = Np)
  i1k3=vector(mode = "double",length = Np)
  i2k3=vector(mode = "double",length = Np)
  r1k3=vector(mode = "double",length = Np)
  r2k3=vector(mode = "double",length = Np)
  ik3a=vector(mode = "double",length = Np)
  nk3=vector(mode = "double",length = Np)
  Ts3=vector(mode = "double",length = Np)
  Ti13=vector(mode = "double",length = Np)
  Ti23=vector(mode = "double",length = Np)
  Tr13=vector(mode = "double",length = Np)
  Tr23=vector(mode = "double",length = Np)
  
  sk4=vector(mode = "double",length = Np)
  i1k4=vector(mode = "double",length = Np)
  i2k4=vector(mode = "double",length = Np)
  r1k4=vector(mode = "double",length = Np)
  r2k4=vector(mode = "double",length = Np)
  ik4a=vector(mode = "double",length = Np)

  cnt=1
  for (t in tm_vec){
    cnt=cnt+1;
    k=floor(t)%%yearday
    if (k==0) k=yearday
    
    #The first order
    for (i in 1:Np){
      Einf[i]=tm_step*(beta[i]*S[i,cnt-1]*(I1[i,cnt-1]+I2[i,cnt-1]))
      Ebac[i]=tm_step*h0*S[i,cnt-1]
      Erec1[i]=tm_step*(r*I1[i,cnt-1])
      Erec2[i]=tm_step*(r*I2[i,cnt-1])
      EmorS[i]=tm_step*(u+uh[i])*S[i,cnt-1]
      EmorI1[i]=tm_step*(u+uh[i])*I1[i,cnt-1]
      Emord[i]=tm_step*ud*I1[i,cnt-1]
      EmorI2[i]=tm_step*(u+uh[i])*I2[i,cnt-1]
      EmorR1[i]=tm_step*(u+uh[i])*R1[i,cnt-1]
      EmorR2[i]=tm_step*(u+uh[i])*R2[i,cnt-1]
      EmigSout[i]=tm_step*m[i,k]*S[i,cnt-1]
      EmigI2out[i]=tm_step*m[i,k]*I2[i,cnt-1]
      EmigR2out[i]=tm_step*m[i,k]*R2[i,cnt-1]
      EmigabI[i]=tm_step*v*I1[i,cnt-1]
      EmigabR[i]=tm_step*v*R1[i,cnt-1]
      Ebirth[i]=tm_step*b[i,k]
    }
    
    Einf[Einf<0]=0
    Ebac[Ebac<0]=0
    Erec1[Erec1<0]=0
    Erec2[Erec2<0]=0
    EmorS[EmorS<0]=0
    EmorI1[EmorI1<0]=0
    Emord[Emord<0]=0
    EmorI2[EmorI2<0]=0
    EmorR1[EmorR1<0]=0
    EmorR2[EmorR2<0]=0
    EmigSout[EmigSout<0]=0
    EmigI2out[EmigI2out<0]=0
    EmigR2out[EmigR2out<0]=0
    EmigabI[EmigabI<0]=0
    EmigabR[EmigabR<0]=0
    Ebirth[Ebirth<0]=0
    
    #Poisson
    for (i in 1:Np){
      stEinf[i]=rpois(1,Einf[i])
      stEbac[i]=rpois(1,Ebac[i])
      stErec1[i]=rpois(1,Erec1[i])
      stErec2[i]=rpois(1,Erec2[i])
      stEmorS[i]=rpois(1,EmorS[i])
      stEmorI1[i]=rpois(1,EmorI1[i])
      stEmord[i]=rpois(1,Emord[i])
      stEmorI2[i]=rpois(1,EmorI2[i])
      stEmorR1[i]=rpois(1,EmorR1[i])
      stEmorR2[i]=rpois(1,EmorR2[i])
      stEmigSout[i]=rpois(1,EmigSout[i])
      stEmigI2out[i]=rpois(1,EmigI2out[i])
      stEmigR2out[i]=rpois(1,EmigR2out[i])
      stEmigabI[i]=rpois(1,EmigabI[i])
      stEmigabR[i]=rpois(1,EmigabR[i])
      stEbirth[i]=rpois(1,Ebirth[i])
    }
    
    for (i in 1:Np){
      if (stEinf[i]>S[i,cnt-1]) stEinf[i]=S[i,cnt-1]
      if (stEbac[i]>S[i,cnt-1]) stEbac[i]=S[i,cnt-1]
      if (stErec1[i]>I1[i,cnt-1]) stErec1[i]=I1[i,cnt-1]
      if (stErec2[i]>I2[i,cnt-1]) stErec2[i]=I2[i,cnt-1]
      if (stEmorS[i]>S[i,cnt-1]) stEmorS[i]=S[i,cnt-1]
      if (stEmorI1[i]>I1[i,cnt-1]) stEmorI1[i]=I1[i,cnt-1]
      if (stEmord[i]>I1[i,cnt-1]) stEmord[i]=I1[i,cnt-1]
      if (stEmorI2[i]>I2[i,cnt-1]) stEmorI2[i]=I2[i,cnt-1]
      if (stEmorR1[i]>R1[i,cnt-1]) stEmorR1[i]=R1[i,cnt-1]
      if (stEmorR2[i]>R2[i,cnt-1]) stEmorR2[i]=R2[i,cnt-1]
      if (stEmigSout[i]>S[i,cnt-1]) stEmigSout[i]=S[i,cnt-1]
      if (stEmigI2out[i]>I2[i,cnt-1]) stEmigI2out[i]=I2[i,cnt-1]
      if (stEmigR2out[i]>R2[i,cnt-1]) stEmigR2out[i]=R2[i,cnt-1]
      if (stEmigabI[i]>I1[i,cnt-1]) stEmigabI[i]=I1[i,cnt-1]
      if (stEmigabR[i]>R1[i,cnt-1]) stEmigabR[i]=R1[i,cnt-1]
    }
    
    for (i in 1:Np){
      if (i==1){
        j=Np
      } else{
        j=i-1
      }
      sk1[i]=-stEinf[i]-stEbac[i]-stEmigSout[i]-stEmorS[i]+stEbirth[i]+stEmigSout[j]
      i1k1[i]=stEinf[i]+stEbac[i]-stEmorI1[i]-stEmord[i]-stEmigabI[i]-stErec1[i]
      i2k1[i]=stEmigabI[i]-stEmigI2out[i]-stEmorI2[i]-stErec2[i]+stEmigI2out[j]
      r1k1[i]=stErec1[i]-stEmorR1[i]-stEmigabR[i]
      r2k1[i]=stErec2[i]-stEmigR2out[i]-stEmorR2[i]+stEmigR2out[j]+stEmigabR[i]
      ik1a[i]=stEinf[i]+stEbac[i]

      Ts1[i]=S[i,cnt-1]+round(sk1[i]/2,0)
      Ti11[i]=I1[i,cnt-1]+round(i1k1[i]/2,0)
      Ti21[i]=I2[i,cnt-1]+round(i2k1[i]/2,0)
      Tr11[i]=R1[i,cnt-1]+round(r1k1[i]/2,0)
      Tr21[i]=R2[i,cnt-1]+round(r2k1[i]/2,0)
      
    }
    
    #The second order
    for (i in 1:Np){
      Einf[i]=tm_step*(beta[i]*Ts1[i]*(Ti11[i]+Ti21[i]))
      Ebac[i]=tm_step*h0*Ts1[i]
      Erec1[i]=tm_step*(r*Ti11[i])
      Erec2[i]=tm_step*(r*Ti21[i])
      EmorS[i]=tm_step*(u+uh[i])*Ts1[i]
      EmorI1[i]=tm_step*(u+uh[i])*Ti11[i]
      Emord[i]=tm_step*ud*Ti11[i]
      EmorI2[i]=tm_step*(u+uh[i])*Ti21[i]
      EmorR1[i]=tm_step*(u+uh[i])*Tr11[i]
      EmorR2[i]=tm_step*(u+uh[i])*Tr21[i]
      EmigSout[i]=tm_step*m[i,k]*Ts1[i]
      EmigI2out[i]=tm_step*m[i,k]*Ti21[i]
      EmigR2out[i]=tm_step*m[i,k]*Tr21[i]
      EmigabI[i]=tm_step*v*Ti11[i]
      EmigabR[i]=tm_step*v*Tr11[i]
      Ebirth[i]=tm_step*b[i,k]
    }
    
    Einf[Einf<0]=0
    Ebac[Ebac<0]=0
    Erec1[Erec1<0]=0
    Erec2[Erec2<0]=0
    EmorS[EmorS<0]=0
    EmorI1[EmorI1<0]=0
    Emord[Emord<0]=0
    EmorI2[EmorI2<0]=0
    EmorR1[EmorR1<0]=0
    EmorR2[EmorR2<0]=0
    EmigSout[EmigSout<0]=0
    EmigI2out[EmigI2out<0]=0
    EmigR2out[EmigR2out<0]=0
    EmigabI[EmigabI<0]=0
    EmigabR[EmigabR<0]=0
    Ebirth[Ebirth<0]=0
    
    #Poisson
    for (i in 1:Np){
      stEinf[i]=rpois(1,Einf[i])
      stEbac[i]=rpois(1,Ebac[i])
      stErec1[i]=rpois(1,Erec1[i])
      stErec2[i]=rpois(1,Erec2[i])
      stEmorS[i]=rpois(1,EmorS[i])
      stEmorI1[i]=rpois(1,EmorI1[i])
      stEmord[i]=rpois(1,Emord[i])
      stEmorI2[i]=rpois(1,EmorI2[i])
      stEmorR1[i]=rpois(1,EmorR1[i])
      stEmorR2[i]=rpois(1,EmorR2[i])
      stEmigSout[i]=rpois(1,EmigSout[i])
      stEmigI2out[i]=rpois(1,EmigI2out[i])
      stEmigR2out[i]=rpois(1,EmigR2out[i])
      stEmigabI[i]=rpois(1,EmigabI[i])
      stEmigabR[i]=rpois(1,EmigabR[i])
      stEbirth[i]=rpois(1,Ebirth[i])
    }
    
    for (i in 1:Np){
      if (stEinf[i]>Ts1[i]) stEinf[i]=Ts1[i]
      if (stEbac[i]>Ts1[i]) stEbac[i]=Ts1[i]
      if (stErec1[i]>Ti11[i]) stErec1[i]=Ti11[i]
      if (stErec2[i]>Ti21[i]) stErec2[i]=Ti21[i]
      if (stEmorS[i]>Ts1[i]) stEmorS[i]=Ts1[i]
      if (stEmorI1[i]>Ti11[i]) stEmorI1[i]=Ti11[i]
      if (stEmord[i]>Ti11[i]) stEmord[i]=Ti11[i]
      if (stEmorI2[i]>Ti21[i]) stEmorI2[i]=Ti21[i]
      if (stEmorR1[i]>Tr11[i]) stEmorR1[i]=Tr11[i]
      if (stEmorR2[i]>Tr21[i]) stEmorR2[i]=Tr21[i]
      if (stEmigSout[i]>Ts1[i]) stEmigSout[i]=Ts1[i]
      if (stEmigI2out[i]>Ti21[i]) stEmigI2out[i]=Ti21[i]
      if (stEmigR2out[i]>Tr21[i]) stEmigR2out[i]=Tr21[i]
      if (stEmigabI[i]>Ti11[i]) stEmigabI[i]=Ti11[i]
      if (stEmigabR[i]>Tr11[i]) stEmigabR[i]=Tr11[i]
    }
    
    for (i in 1:Np){
      if (i==1){
        j=Np
      } else{
        j=i-1
      }
      sk2[i]=-stEinf[i]-stEbac[i]-stEmigSout[i]-stEmorS[i]+stEbirth[i]+stEmigSout[j]
      i1k2[i]=stEinf[i]+stEbac[i]-stEmorI1[i]-stEmord[i]-stEmigabI[i]-stErec1[i]
      i2k2[i]=stEmigabI[i]-stEmigI2out[i]-stEmorI2[i]-stErec2[i]+stEmigI2out[j]
      r1k2[i]=stErec1[i]-stEmorR1[i]-stEmigabR[i]
      r2k2[i]=stErec2[i]-stEmigR2out[i]-stEmorR2[i]+stEmigR2out[j]+stEmigabR[i]
      ik2a[i]=stEinf[i]+stEbac[i]

      Ts2[i]=S[i,cnt-1]+round(sk2[i]/2,0)
      Ti12[i]=I1[i,cnt-1]+round(i1k2[i]/2,0)
      Ti22[i]=I2[i,cnt-1]+round(i2k2[i]/2,0)
      Tr12[i]=R1[i,cnt-1]+round(r1k2[i]/2,0)
      Tr22[i]=R2[i,cnt-1]+round(r2k2[i]/2,0)
    }
    
    #The third order
    for (i in 1:Np){
      Einf[i]=tm_step*(beta[i]*Ts2[i]*(Ti12[i]+Ti22[i]))
      Ebac[i]=tm_step*h0*Ts2[i]
      Erec1[i]=tm_step*(r*Ti12[i])
      Erec2[i]=tm_step*(r*Ti22[i])
      EmorS[i]=tm_step*(u+uh[i])*Ts2[i]
      EmorI1[i]=tm_step*(u+uh[i])*Ti12[i]
      Emord[i]=tm_step*ud*Ti12[i]
      EmorI2[i]=tm_step*(u+uh[i])*Ti22[i]
      EmorR1[i]=tm_step*(u+uh[i])*Tr12[i]
      EmorR2[i]=tm_step*(u+uh[i])*Tr22[i]
      EmigSout[i]=tm_step*m[i,k]*Ts2[i]
      EmigI2out[i]=tm_step*m[i,k]*Ti22[i]
      EmigR2out[i]=tm_step*m[i,k]*Tr22[i]
      EmigabI[i]=tm_step*v*Ti12[i]
      EmigabR[i]=tm_step*v*Tr12[i]
      Ebirth[i]=tm_step*b[i,k]
    }
    
    Einf[Einf<0]=0
    Ebac[Ebac<0]=0
    Erec1[Erec1<0]=0
    Erec2[Erec2<0]=0
    EmorS[EmorS<0]=0
    EmorI1[EmorI1<0]=0
    Emord[Emord<0]=0
    EmorI2[EmorI2<0]=0
    EmorR1[EmorR1<0]=0
    EmorR2[EmorR2<0]=0
    EmigSout[EmigSout<0]=0
    EmigI2out[EmigI2out<0]=0
    EmigR2out[EmigR2out<0]=0
    EmigabI[EmigabI<0]=0
    EmigabR[EmigabR<0]=0
    Ebirth[Ebirth<0]=0
    
    #Poisson
    for (i in 1:Np){
      stEinf[i]=rpois(1,Einf[i])
      stEbac[i]=rpois(1,Ebac[i])
      stErec1[i]=rpois(1,Erec1[i])
      stErec2[i]=rpois(1,Erec2[i])
      stEmorS[i]=rpois(1,EmorS[i])
      stEmorI1[i]=rpois(1,EmorI1[i])
      stEmord[i]=rpois(1,Emord[i])
      stEmorI2[i]=rpois(1,EmorI2[i])
      stEmorR1[i]=rpois(1,EmorR1[i])
      stEmorR2[i]=rpois(1,EmorR2[i])
      stEmigSout[i]=rpois(1,EmigSout[i])
      stEmigI2out[i]=rpois(1,EmigI2out[i])
      stEmigR2out[i]=rpois(1,EmigR2out[i])
      stEmigabI[i]=rpois(1,EmigabI[i])
      stEmigabR[i]=rpois(1,EmigabR[i])
      stEbirth[i]=rpois(1,Ebirth[i])
    }
    
    for (i in 1:Np){
      if (stEinf[i]>Ts2[i]) stEinf[i]=Ts2[i]
      if (stEbac[i]>Ts2[i]) stEbac[i]=Ts2[i]
      if (stErec1[i]>Ti12[i]) stErec1[i]=Ti12[i]
      if (stErec2[i]>Ti22[i]) stErec2[i]=Ti22[i]
      if (stEmorS[i]>Ts2[i]) stEmorS[i]=Ts2[i]
      if (stEmorI1[i]>Ti12[i]) stEmorI1[i]=Ti12[i]
      if (stEmord[i]>Ti12[i]) stEmord[i]=Ti12[i]
      if (stEmorI2[i]>Ti22[i]) stEmorI2[i]=Ti22[i]
      if (stEmorR1[i]>Tr12[i]) stEmorR1[i]=Tr12[i]
      if (stEmorR2[i]>Tr22[i]) stEmorR2[i]=Tr22[i]
      if (stEmigSout[i]>Ts2[i]) stEmigSout[i]=Ts2[i]
      if (stEmigI2out[i]>Ti22[i]) stEmigI2out[i]=Ti22[i]
      if (stEmigR2out[i]>Tr22[i]) stEmigR2out[i]=Tr22[i]
      if (stEmigabI[i]>Ti12[i]) stEmigabI[i]=Ti12[i]
      if (stEmigabR[i]>Tr12[i]) stEmigabR[i]=Tr12[i]
    }
    
    for (i in 1:Np){
      if (i==1){
        j=Np
      } else{
        j=i-1
      }
      sk3[i]=-stEinf[i]-stEbac[i]-stEmigSout[i]-stEmorS[i]+stEbirth[i]+stEmigSout[j]
      i1k3[i]=stEinf[i]+stEbac[i]-stEmorI1[i]-stEmord[i]-stEmigabI[i]-stErec1[i]
      i2k3[i]=stEmigabI[i]-stEmigI2out[i]-stEmorI2[i]-stErec2[i]+stEmigI2out[j]
      r1k3[i]=stErec1[i]-stEmorR1[i]-stEmigabR[i]
      r2k3[i]=stErec2[i]-stEmigR2out[i]-stEmorR2[i]+stEmigR2out[j]+stEmigabR[i]
      ik3a[i]=stEinf[i]+stEbac[i]

      Ts3[i]=S[i,cnt-1]+round(sk3[i],0)
      Ti13[i]=I1[i,cnt-1]+round(i1k3[i],0)
      Ti23[i]=I2[i,cnt-1]+round(i2k3[i],0)
      Tr13[i]=R1[i,cnt-1]+round(r1k3[i],0)
      Tr23[i]=R2[i,cnt-1]+round(r2k3[i],0)
    }
    
    #The forth order
    for (i in 1:Np){
      Einf[i]=tm_step*(beta[i]*Ts3[i]*(Ti13[i]+Ti23[i]))
      Ebac[i]=tm_step*h*Ts3[i]
      Erec1[i]=tm_step*(r*Ti13[i])
      Erec2[i]=tm_step*(r*Ti23[i])
      EmorS[i]=tm_step*(u+uh[i])*Ts3[i]
      EmorI1[i]=tm_step*(u+uh[i])*Ti13[i]
      Emord[i]=tm_step*ud*Ti13[i]
      EmorI2[i]=tm_step*(u+uh[i])*Ti23[i]
      EmorR1[i]=tm_step*(u+uh[i])*Tr13[i]
      EmorR2[i]=tm_step*(u+uh[i])*Tr23[i]
      EmigSout[i]=tm_step*m[i,k]*Ts3[i]
      EmigI2out[i]=tm_step*m[i,k]*Ti23[i]
      EmigR2out[i]=tm_step*m[i,k]*Tr23[i]
      EmigabI[i]=tm_step*v*Ti13[i]
      EmigabR[i]=tm_step*v*Tr13[i]
      Ebirth[i]=tm_step*b[i,k]
    }
    
    Einf[Einf<0]=0
    Ebac[Ebac<0]=0
    Erec1[Erec1<0]=0
    Erec2[Erec2<0]=0
    EmorS[EmorS<0]=0
    EmorI1[EmorI1<0]=0
    Emord[Emord<0]=0
    EmorI2[EmorI2<0]=0
    EmorR1[EmorR1<0]=0
    EmorR2[EmorR2<0]=0
    EmigSout[EmigSout<0]=0
    EmigI2out[EmigI2out<0]=0
    EmigR2out[EmigR2out<0]=0
    EmigabI[EmigabI<0]=0
    EmigabR[EmigabR<0]=0
    Ebirth[Ebirth<0]=0
    
    #Poisson
    for (i in 1:Np){
      stEinf[i]=rpois(1,Einf[i])
      stEbac[i]=rpois(1,Ebac[i])
      stErec1[i]=rpois(1,Erec1[i])
      stErec2[i]=rpois(1,Erec2[i])
      stEmorS[i]=rpois(1,EmorS[i])
      stEmorI1[i]=rpois(1,EmorI1[i])
      stEmord[i]=rpois(1,Emord[i])
      stEmorI2[i]=rpois(1,EmorI2[i])
      stEmorR1[i]=rpois(1,EmorR1[i])
      stEmorR2[i]=rpois(1,EmorR2[i])
      stEmigSout[i]=rpois(1,EmigSout[i])
      stEmigI2out[i]=rpois(1,EmigI2out[i])
      stEmigR2out[i]=rpois(1,EmigR2out[i])
      stEmigabI[i]=rpois(1,EmigabI[i])
      stEmigabR[i]=rpois(1,EmigabR[i])
      stEbirth[i]=rpois(1,Ebirth[i])
    }
    
    for (i in 1:Np){
      if (stEinf[i]>Ts3[i]) stEinf[i]=Ts3[i]
      if (stEbac[i]>Ts3[i]) stEbac[i]=Ts3[i]
      if (stErec1[i]>Ti13[i]) stErec1[i]=Ti13[i]
      if (stErec2[i]>Ti23[i]) stErec2[i]=Ti23[i]
      if (stEmorS[i]>Ts3[i]) stEmorS[i]=Ts3[i]
      if (stEmorI1[i]>Ti13[i]) stEmorI1[i]=Ti13[i]
      if (stEmord[i]>Ti13[i]) stEmord[i]=Ti13[i]
      if (stEmorI2[i]>Ti23[i]) stEmorI2[i]=Ti23[i]
      if (stEmorR1[i]>Tr13[i]) stEmorR1[i]=Tr13[i]
      if (stEmorR2[i]>Tr23[i]) stEmorR2[i]=Tr23[i]
      if (stEmigSout[i]>Ts3[i]) stEmigSout[i]=Ts3[i]
      if (stEmigI2out[i]>Ti23[i]) stEmigI2out[i]=Ti23[i]
      if (stEmigR2out[i]>Tr23[i]) stEmigR2out[i]=Tr23[i]
      if (stEmigabI[i]>Ti13[i]) stEmigabI[i]=Ti13[i]
      if (stEmigabR[i]>Tr13[i]) stEmigabR[i]=Tr13[i]
    }
    
    for (i in 1:Np){
      if (i==1){
        j=Np
      } else{
        j=i-1
      }
      sk4[i]=-stEinf[i]-stEbac[i]-stEmigSout[i]-stEmorS[i]+stEbirth[i]+stEmigSout[j]
      i1k4[i]=stEinf[i]+stEbac[i]-stEmorI1[i]-stEmord[i]-stEmigabI[i]-stErec1[i]
      i2k4[i]=stEmigabI[i]-stEmigI2out[i]-stEmorI2[i]-stErec2[i]+stEmigI2out[j]
      r1k4[i]=stErec1[i]-stEmorR1[i]-stEmigabR[i]
      r2k4[i]=stErec2[i]-stEmigR2out[i]-stEmorR2[i]+stEmigR2out[j]+stEmigabR[i]
      ik4a[i]=stEinf[i]+stEbac[i]
    }
    
    for (i in 1:Np){ 
      S[i,cnt]=S[i,cnt-1]+round(sk1[i]/6+sk2[i]/3+sk3[i]/3+sk4[i]/6)
      I1[i,cnt]=I1[i,cnt-1]+round(i1k1[i]/6+i1k2[i]/3+i1k3[i]/3+i1k4[i]/6)
      I2[i,cnt]=I2[i,cnt-1]+round(i2k1[i]/6+i2k2[i]/3+i2k3[i]/3+i2k4[i]/6)
      R1[i,cnt]=R1[i,cnt-1]+round(r1k1[i]/6+r1k2[i]/3+r1k3[i]/3+r1k4[i]/6)
      R2[i,cnt]=R2[i,cnt-1]+round(r2k1[i]/6+r2k2[i]/3+r2k3[i]/3+r2k4[i]/6)
      newI[i,cnt]=newI[i,cnt-1]+round(ik1a[i]/6+ik2a[i]/3+ik3a[i]/3+ik4a[i]/6)
    }
    
    #for (i in 1:Np){
    #I=I+I1[i,cnt]+I2[i,cnt]
    #}
    #if (I==0) break
    
  }
  
  rec=list(S=S,I=I1+I2,R=R1+R2); 
  
  
  rec;
  
}

#Function to simulate virus transmission in a migratory population with environmental transmission (the secod model)
migdelayStoSIIRREnv<-function(yearday, year, tm_step, S0, I0, E0, N, r, beta, u, uh, ud, b, m, Np, v, c, gamma, alpha, realdata=FALSE){
  # Input: yearday: days of a year; year: years; tm_step: time step;
  #         S0, I0: initial states matrix; N: population size; E0: initial virions in the environment
  #         r: infection recovery rate, day; beta: contact transmission matrix;
  #         u: natural mortality rate; uh: hunting mortality rate; b: birth rate matrix;
  #         m: migration rate matrix
  #         NP: patch number
  #         v: migration recovery rate
  #         c: persistence of virions in the environment
  #         gamma: exposure rate
  #         alpha: re-scaled environmental infectiousness
  #
  # output: S, I, R, E for all time steps
  cnt=1
  tm_vec=seq(1,year*yearday+1,by=tm_step)
  tm_sz=length(tm_vec)+1;
  #Np=length(S0)# number of patches
  S=I1=I2=R1=R2=E=matrix(0,Np,tm_sz)
  
  S[,1]=round(S0,0); I1[,1]=round(I0,0); R1[,1]=round((N-S0-I0),0); E[,1]=E0
  #temperal parameters
  Einf=vector(mode = "double",length = Np)
  Eenv=vector(mode = "double",length = Np)
  Erec1=vector(mode = "double",length = Np)
  Erec2=vector(mode = "double",length = Np)
  EmorS=vector(mode = "double",length = Np)
  EmorI1=vector(mode = "double",length = Np)
  EmorI2=vector(mode = "double",length = Np)
  EmorR1=vector(mode = "double",length = Np)
  EmorR2=vector(mode = "double",length = Np)
  Emord=vector(mode = "double",length = Np)
  EmigSout=vector(mode = "double",length = Np)
  EmigI2out=vector(mode = "double",length = Np)
  EmigR2out=vector(mode = "double",length = Np)
  EmigabI=vector(mode = "double",length = Np)
  EmigabR=vector(mode = "double",length = Np)
  Ebirth=vector(mode = "double",length = Np)
  Eper=vector(mode = "double",length = Np)
  EI=vector(mode = "double",length = Np)
  
  stEinf=vector(mode = "double",length = Np)
  stEenv=vector(mode = "double",length = Np)
  stErec1=vector(mode = "double",length = Np)
  stErec2=vector(mode = "double",length = Np)
  stEmorS=vector(mode = "double",length = Np)
  stEmorI1=vector(mode = "double",length = Np)
  stEmorI2=vector(mode = "double",length = Np)
  stEmorR1=vector(mode = "double",length = Np)
  stEmorR2=vector(mode = "double",length = Np)
  stEmord=vector(mode = "double",length = Np)
  stEmigSout=vector(mode = "double",length = Np)
  stEmigI2out=vector(mode = "double",length = Np)
  stEmigR2out=vector(mode = "double",length = Np)
  stEmigabI=vector(mode = "double",length = Np)
  stEmigabR=vector(mode = "double",length = Np)
  stEbirth=vector(mode = "double",length = Np)
  stEper=vector(mode = "double",length = Np)
  stEI=vector(mode = "double",length = Np)
  
  sk1=vector(mode = "double",length = Np)
  i1k1=vector(mode = "double",length = Np)
  i2k1=vector(mode = "double",length = Np)
  r1k1=vector(mode = "double",length = Np)
  r2k1=vector(mode = "double",length = Np)
  ek1=vector(mode = "double",length = Np)
  Ts1=vector(mode = "double",length = Np)
  Ti11=vector(mode = "double",length = Np)
  Ti21=vector(mode = "double",length = Np)
  Tr11=vector(mode = "double",length = Np)
  Tr21=vector(mode = "double",length = Np)
  Te1=vector(mode = "double",length = Np)
  
  sk2=vector(mode = "double",length = Np)
  i1k2=vector(mode = "double",length = Np)
  i2k2=vector(mode = "double",length = Np)
  r1k2=vector(mode = "double",length = Np)
  r2k2=vector(mode = "double",length = Np)
  ek2=vector(mode = "double",length = Np)
  Ts2=vector(mode = "double",length = Np)
  Ti12=vector(mode = "double",length = Np)
  Ti22=vector(mode = "double",length = Np)
  Tr12=vector(mode = "double",length = Np)
  Tr22=vector(mode = "double",length = Np)
  Te2=vector(mode = "double",length = Np)
  
  
  sk3=vector(mode = "double",length = Np)
  i1k3=vector(mode = "double",length = Np)
  i2k3=vector(mode = "double",length = Np)
  r1k3=vector(mode = "double",length = Np)
  r2k3=vector(mode = "double",length = Np)
  ek3=vector(mode = "double",length = Np)
  Ts3=vector(mode = "double",length = Np)
  Ti13=vector(mode = "double",length = Np)
  Ti23=vector(mode = "double",length = Np)
  Tr13=vector(mode = "double",length = Np)
  Tr23=vector(mode = "double",length = Np)
  Te3=vector(mode = "double",length = Np)
  
  
  sk4=vector(mode = "double",length = Np)
  i1k4=vector(mode = "double",length = Np)
  i2k4=vector(mode = "double",length = Np)
  r1k4=vector(mode = "double",length = Np)
  r2k4=vector(mode = "double",length = Np)
  ek4=vector(mode = "double",length = Np)

  for (t in tm_vec){
    cnt=cnt+1;
    k=floor(t)%%yearday
    if (k==0) k=yearday
    
    #The first order
    for (i in 1:Np){
      Einf[i]=tm_step*(beta[i]*S[i,cnt-1]*(I1[i,cnt-1]+I2[i,cnt-1]))
      Eenv[i]=tm_step*(gamma*S[i,cnt-1]*(1-exp(-alpha*E[i,cnt-1])))
      Erec1[i]=tm_step*(r*I1[i,cnt-1])
      Erec2[i]=tm_step*(r*I2[i,cnt-1])
      EmorS[i]=tm_step*(u+uh[i])*S[i,cnt-1]
      EmorI1[i]=tm_step*(u+uh[i])*I1[i,cnt-1]
      Emord[i]=tm_step*ud*I1[i,cnt-1]
      EmorI2[i]=tm_step*(u+uh[i])*I2[i,cnt-1]
      EmorR1[i]=tm_step*(u+uh[i])*R1[i,cnt-1]
      EmorR2[i]=tm_step*(u+uh[i])*R2[i,cnt-1]
      EmigSout[i]=tm_step*m[i,k]*S[i,cnt-1]
      EmigI2out[i]=tm_step*m[i,k]*I2[i,cnt-1]
      EmigR2out[i]=tm_step*m[i,k]*R2[i,cnt-1]
      EmigabI[i]=tm_step*v*I1[i,cnt-1]
      EmigabR[i]=tm_step*v*R1[i,cnt-1]
      Ebirth[i]=tm_step*b[i,k]
      Eper[i]=tm_step*c*E[i,cnt-1]
      EI[i]=tm_step*(I1[i,cnt-1]+I2[i,cnt-1])
    }
    
    Einf[Einf<0]=0
    Eenv[Eenv<0]=0
    Erec1[Erec1<0]=0
    Erec2[Erec2<0]=0
    EmorS[EmorS<0]=0
    EmorI1[EmorI1<0]=0
    Emord[Emord<0]=0
    EmorI2[EmorI2<0]=0
    EmorR1[EmorR1<0]=0
    EmorR2[EmorR2<0]=0
    EmigSout[EmigSout<0]=0
    EmigI2out[EmigI2out<0]=0
    EmigR2out[EmigR2out<0]=0
    EmigabI[EmigabI<0]=0
    EmigabR[EmigabR<0]=0
    Ebirth[Ebirth<0]=0
    EI[EI<0]=0
    Eper[Eper<0]=0
    
    Einf[which(is.na(Einf))]=0
    Eenv[which(is.na(Eenv))]=0
    Erec1[which(is.na(Erec1))]=0
    Erec2[which(is.na(Erec2))]=0
    EmorS[which(is.na(EmorS))]=0
    EmorI1[which(is.na(EmorI1))]=0
    Emord[which(is.na(Emord))]=0
    EmorI2[which(is.na(EmorI2))]=0
    EmorR1[which(is.na(EmorR1))]=0
    EmorR2[which(is.na(EmorR2))]=0
    EmigSout[which(is.na(EmigSout))]=0
    EmigI2out[which(is.na(EmigI2out))]=0
    EmigR2out[which(is.na(EmigR2out))]=0
    EmigabI[which(is.na(EmigabI))]=0
    EmigabR[which(is.na(EmigabR))]=0
    Ebirth[which(is.na(Ebirth))]=0
    EI[which(is.na(EI))]=0
    Eper[which(is.na(Eper))]=0
    
    #Poisson
    for (i in 1:Np){
      stEinf[i]=rpois(1,Einf[i])
      stEenv[i]=rpois(1,Eenv[i])
      stErec1[i]=rpois(1,Erec1[i])
      stErec2[i]=rpois(1,Erec2[i])
      stEmorS[i]=rpois(1,EmorS[i])
      stEmorI1[i]=rpois(1,EmorI1[i])
      stEmord[i]=rpois(1,Emord[i])
      stEmorI2[i]=rpois(1,EmorI2[i])
      stEmorR1[i]=rpois(1,EmorR1[i])
      stEmorR2[i]=rpois(1,EmorR2[i])
      stEmigSout[i]=rpois(1,EmigSout[i])
      stEmigI2out[i]=rpois(1,EmigI2out[i])
      stEmigR2out[i]=rpois(1,EmigR2out[i])
      stEmigabI[i]=rpois(1,EmigabI[i])
      stEmigabR[i]=rpois(1,EmigabR[i])
      stEbirth[i]=rpois(1,Ebirth[i])
      stEper[i]=rpois(1,Eper[i])
      stEI[i]=rpois(1,EI[i])
    }
    
    for (i in 1:Np){
      if (stEinf[i]>S[i,cnt-1]) stEinf[i]=S[i,cnt-1]
      if (stEenv[i]>S[i,cnt-1]) stEenv[i]=S[i,cnt-1]
      if (stErec1[i]>I1[i,cnt-1]) stErec1[i]=I1[i,cnt-1]
      if (stErec2[i]>I2[i,cnt-1]) stErec2[i]=I2[i,cnt-1]
      if (stEmorS[i]>S[i,cnt-1]) stEmorS[i]=S[i,cnt-1]
      if (stEmorI1[i]>I1[i,cnt-1]) stEmorI1[i]=I1[i,cnt-1]
      if (stEmord[i]>I1[i,cnt-1]) stEmord[i]=I1[i,cnt-1]
      if (stEmorI2[i]>I2[i,cnt-1]) stEmorI2[i]=I2[i,cnt-1]
      if (stEmorR1[i]>R1[i,cnt-1]) stEmorR1[i]=R1[i,cnt-1]
      if (stEmorR2[i]>R2[i,cnt-1]) stEmorR2[i]=R2[i,cnt-1]
      if (stEmigSout[i]>S[i,cnt-1]) stEmigSout[i]=S[i,cnt-1]
      if (stEmigI2out[i]>I2[i,cnt-1]) stEmigI2out[i]=I2[i,cnt-1]
      if (stEmigR2out[i]>R2[i,cnt-1]) stEmigR2out[i]=R2[i,cnt-1]
      if (stEmigabI[i]>I1[i,cnt-1]) stEmigabI[i]=I1[i,cnt-1]
      if (stEmigabR[i]>R1[i,cnt-1]) stEmigabR[i]=R1[i,cnt-1]
      if (stEper[i]>E[i,cnt-1]) stEper[i]=E[i,cnt-1]
    }
    
    for (i in 1:Np){
      if (i==1){
        j=Np
      } else{
        j=i-1
      }
      sk1[i]=-stEinf[i]-stEenv[i]-stEmigSout[i]-stEmorS[i]+stEbirth[i]+stEmigSout[j]
      i1k1[i]=stEinf[i]+stEenv[i]-stEmorI1[i]-stEmord[i]-stEmigabI[i]-stErec1[i]
      i2k1[i]=stEmigabI[i]-stEmigI2out[i]-stEmorI2[i]-stErec2[i]+stEmigI2out[j]
      r1k1[i]=stErec1[i]-stEmorR1[i]-stEmigabR[i]
      r2k1[i]=stErec2[i]-stEmigR2out[i]-stEmorR2[i]+stEmigR2out[j]+stEmigabR[i]
      ek1[i]=stEI[i]-stEper[i]

      Ts1[i]=S[i,cnt-1]+round(sk1[i]/2,0)
      Ti11[i]=I1[i,cnt-1]+round(i1k1[i]/2,0)
      Ti21[i]=I2[i,cnt-1]+round(i2k1[i]/2,0)
      Tr11[i]=R1[i,cnt-1]+round(r1k1[i]/2,0)
      Tr21[i]=R2[i,cnt-1]+round(r2k1[i]/2,0)
      Te1[i]=E[i,cnt-1]+round(ek1[i]/2,0)
      
    }
    
    #The second order
    for (i in 1:Np){
      Einf[i]=tm_step*(beta[i]*Ts1[i]*(Ti11[i]+Ti21[i]))
      Eenv[i]=tm_step*(gamma*Ts1[i]*(1-exp(-alpha*Te1[i])))
      Erec1[i]=tm_step*(r*Ti11[i])
      Erec2[i]=tm_step*(r*Ti21[i])
      EmorS[i]=tm_step*(u+uh[i])*Ts1[i]
      EmorI1[i]=tm_step*(u+uh[i])*Ti11[i]
      Emord[i]=tm_step*ud*Ti11[i]
      EmorI2[i]=tm_step*(u+uh[i])*Ti21[i]
      EmorR1[i]=tm_step*(u+uh[i])*Tr11[i]
      EmorR2[i]=tm_step*(u+uh[i])*Tr21[i]
      EmigSout[i]=tm_step*m[i,k]*Ts1[i]
      EmigI2out[i]=tm_step*m[i,k]*Ti21[i]
      EmigR2out[i]=tm_step*m[i,k]*Tr21[i]
      EmigabI[i]=tm_step*v*Ti11[i]
      EmigabR[i]=tm_step*v*Tr11[i]
      Ebirth[i]=tm_step*b[i,k]
      Eper[i]=tm_step*c*Te1[i]
      EI[i]=tm_step*(Ti11[i]+Ti21[i])
    }
    
    Einf[Einf<0]=0
    Eenv[Eenv<0]=0
    Erec1[Erec1<0]=0
    Erec2[Erec2<0]=0
    EmorS[EmorS<0]=0
    EmorI1[EmorI1<0]=0
    Emord[Emord<0]=0
    EmorI2[EmorI2<0]=0
    EmorR1[EmorR1<0]=0
    EmorR2[EmorR2<0]=0
    EmigSout[EmigSout<0]=0
    EmigI2out[EmigI2out<0]=0
    EmigR2out[EmigR2out<0]=0
    EmigabI[EmigabI<0]=0
    EmigabR[EmigabR<0]=0
    Ebirth[Ebirth<0]=0
    EI[EI<0]=0
    Eper[Eper<0]=0
    
    Einf[which(is.na(Einf))]=0
    Eenv[which(is.na(Eenv))]=0
    Erec1[which(is.na(Erec1))]=0
    Erec2[which(is.na(Erec2))]=0
    EmorS[which(is.na(EmorS))]=0
    EmorI1[which(is.na(EmorI1))]=0
    Emord[which(is.na(Emord))]=0
    EmorI2[which(is.na(EmorI2))]=0
    EmorR1[which(is.na(EmorR1))]=0
    EmorR2[which(is.na(EmorR2))]=0
    EmigSout[which(is.na(EmigSout))]=0
    EmigI2out[which(is.na(EmigI2out))]=0
    EmigR2out[which(is.na(EmigR2out))]=0
    EmigabI[which(is.na(EmigabI))]=0
    EmigabR[which(is.na(EmigabR))]=0
    Ebirth[which(is.na(Ebirth))]=0
    EI[which(is.na(EI))]=0
    Eper[which(is.na(Eper))]=0
    
    #Poisson
    for (i in 1:Np){
      stEinf[i]=rpois(1,Einf[i])
      stEenv[i]=rpois(1,Eenv[i])
      stErec1[i]=rpois(1,Erec1[i])
      stErec2[i]=rpois(1,Erec2[i])
      stEmorS[i]=rpois(1,EmorS[i])
      stEmorI1[i]=rpois(1,EmorI1[i])
      stEmord[i]=rpois(1,Emord[i])
      stEmorI2[i]=rpois(1,EmorI2[i])
      stEmorR1[i]=rpois(1,EmorR1[i])
      stEmorR2[i]=rpois(1,EmorR2[i])
      stEmigSout[i]=rpois(1,EmigSout[i])
      stEmigI2out[i]=rpois(1,EmigI2out[i])
      stEmigR2out[i]=rpois(1,EmigR2out[i])
      stEmigabI[i]=rpois(1,EmigabI[i])
      stEmigabR[i]=rpois(1,EmigabR[i])
      stEbirth[i]=rpois(1,Ebirth[i])
      stEI[i]=rpois(1,EI[i])
      stEper[i]=rpois(1,Eper[i])
    }
    
    for (i in 1:Np){
      if (stEinf[i]>Ts1[i]) stEinf[i]=Ts1[i]
      if (stEenv[i]>Ts1[i]) stEenv[i]=Ts1[i]
      if (stErec1[i]>Ti11[i]) stErec1[i]=Ti11[i]
      if (stErec2[i]>Ti21[i]) stErec2[i]=Ti21[i]
      if (stEmorS[i]>Ts1[i]) stEmorS[i]=Ts1[i]
      if (stEmorI1[i]>Ti11[i]) stEmorI1[i]=Ti11[i]
      if (stEmord[i]>Ti11[i]) stEmord[i]=Ti11[i]
      if (stEmorI2[i]>Ti21[i]) stEmorI2[i]=Ti21[i]
      if (stEmorR1[i]>Tr11[i]) stEmorR1[i]=Tr11[i]
      if (stEmorR2[i]>Tr21[i]) stEmorR2[i]=Tr21[i]
      if (stEmigSout[i]>Ts1[i]) stEmigSout[i]=Ts1[i]
      if (stEmigI2out[i]>Ti21[i]) stEmigI2out[i]=Ti21[i]
      if (stEmigR2out[i]>Tr21[i]) stEmigR2out[i]=Tr21[i]
      if (stEmigabI[i]>Ti11[i]) stEmigabI[i]=Ti11[i]
      if (stEmigabR[i]>Tr11[i]) stEmigabR[i]=Tr11[i]
      if (stEper[i]>Te1[i]) stEper[i]=Te1[i]
    }
    
    for (i in 1:Np){
      if (i==1){
        j=Np
      } else{
        j=i-1
      }
      sk2[i]=-stEinf[i]-stEenv[i]-stEmigSout[i]-stEmorS[i]+stEbirth[i]+stEmigSout[j]
      i1k2[i]=stEinf[i]+stEenv[i]-stEmorI1[i]-stEmord[i]-stEmigabI[i]-stErec1[i]
      i2k2[i]=stEmigabI[i]-stEmigI2out[i]-stEmorI2[i]-stErec2[i]+stEmigI2out[j]
      r1k2[i]=stErec1[i]-stEmorR1[i]-stEmigabR[i]
      r2k2[i]=stErec2[i]-stEmigR2out[i]-stEmorR2[i]+stEmigR2out[j]+stEmigabR[i]
      ek2[i]=stEI[i]-stEper[i]

      Ts2[i]=S[i,cnt-1]+round(sk2[i]/2,0)
      Ti12[i]=I1[i,cnt-1]+round(i1k2[i]/2,0)
      Ti22[i]=I2[i,cnt-1]+round(i2k2[i]/2,0)
      Tr12[i]=R1[i,cnt-1]+round(r1k2[i]/2,0)
      Tr22[i]=R2[i,cnt-1]+round(r2k2[i]/2,0)
      Te2[i]=E[i,cnt-1]+round(ek2[i]/2,0)
    }
    
    #The third order
    for (i in 1:Np){
      Einf[i]=tm_step*(beta[i]*Ts2[i]*(Ti12[i]+Ti22[i]))
      Eenv[i]=tm_step*(gamma*Ts2[i]*(1-exp(-alpha*Te2[i])))
      Erec1[i]=tm_step*(r*Ti12[i])
      Erec2[i]=tm_step*(r*Ti22[i])
      EmorS[i]=tm_step*(u+uh[i])*Ts2[i]
      EmorI1[i]=tm_step*(u+uh[i])*Ti12[i]
      Emord[i]=tm_step*ud*Ti12[i]
      EmorI2[i]=tm_step*(u+uh[i])*Ti22[i]
      EmorR1[i]=tm_step*(u+uh[i])*Tr12[i]
      EmorR2[i]=tm_step*(u+uh[i])*Tr22[i]
      EmigSout[i]=tm_step*m[i,k]*Ts2[i]
      EmigI2out[i]=tm_step*m[i,k]*Ti22[i]
      EmigR2out[i]=tm_step*m[i,k]*Tr22[i]
      EmigabI[i]=tm_step*v*Ti12[i]
      EmigabR[i]=tm_step*v*Tr12[i]
      Ebirth[i]=tm_step*b[i,k]
      Eper[i]=tm_step*c*Te2[i]
      EI[i]=tm_step*(Ti12[i]+Ti22[i])
    }
    
    Einf[Einf<0]=0
    Eenv[Eenv<0]=0
    Erec1[Erec1<0]=0
    Erec2[Erec2<0]=0
    EmorS[EmorS<0]=0
    EmorI1[EmorI1<0]=0
    Emord[Emord<0]=0
    EmorI2[EmorI2<0]=0
    EmorR1[EmorR1<0]=0
    EmorR2[EmorR2<0]=0
    EmigSout[EmigSout<0]=0
    EmigI2out[EmigI2out<0]=0
    EmigR2out[EmigR2out<0]=0
    EmigabI[EmigabI<0]=0
    EmigabR[EmigabR<0]=0
    Ebirth[Ebirth<0]=0
    EI[EI<0]=0
    Eper[Eper<0]=0
    
    Einf[which(is.na(Einf))]=0
    Eenv[which(is.na(Eenv))]=0
    Erec1[which(is.na(Erec1))]=0
    Erec2[which(is.na(Erec2))]=0
    EmorS[which(is.na(EmorS))]=0
    EmorI1[which(is.na(EmorI1))]=0
    Emord[which(is.na(Emord))]=0
    EmorI2[which(is.na(EmorI2))]=0
    EmorR1[which(is.na(EmorR1))]=0
    EmorR2[which(is.na(EmorR2))]=0
    EmigSout[which(is.na(EmigSout))]=0
    EmigI2out[which(is.na(EmigI2out))]=0
    EmigR2out[which(is.na(EmigR2out))]=0
    EmigabI[which(is.na(EmigabI))]=0
    EmigabR[which(is.na(EmigabR))]=0
    Ebirth[which(is.na(Ebirth))]=0
    EI[which(is.na(EI))]=0
    Eper[which(is.na(Eper))]=0
    
    #Poisson
    for (i in 1:Np){
      stEinf[i]=rpois(1,Einf[i])
      stEenv[i]=rpois(1,Eenv[i])
      stErec1[i]=rpois(1,Erec1[i])
      stErec2[i]=rpois(1,Erec2[i])
      stEmorS[i]=rpois(1,EmorS[i])
      stEmorI1[i]=rpois(1,EmorI1[i])
      stEmord[i]=rpois(1,Emord[i])
      stEmorI2[i]=rpois(1,EmorI2[i])
      stEmorR1[i]=rpois(1,EmorR1[i])
      stEmorR2[i]=rpois(1,EmorR2[i])
      stEmigSout[i]=rpois(1,EmigSout[i])
      stEmigI2out[i]=rpois(1,EmigI2out[i])
      stEmigR2out[i]=rpois(1,EmigR2out[i])
      stEmigabI[i]=rpois(1,EmigabI[i])
      stEmigabR[i]=rpois(1,EmigabR[i])
      stEbirth[i]=rpois(1,Ebirth[i])
      stEI[i]=rpois(1,EI[i])
      stEper[i]=rpois(1,Eper[i])
    }
    
    for (i in 1:Np){
      if (stEinf[i]>Ts2[i]) stEinf[i]=Ts2[i]
      if (stEenv[i]>Ts2[i]) stEenv[i]=Ts2[i]
      if (stErec1[i]>Ti12[i]) stErec1[i]=Ti12[i]
      if (stErec2[i]>Ti22[i]) stErec2[i]=Ti22[i]
      if (stEmorS[i]>Ts2[i]) stEmorS[i]=Ts2[i]
      if (stEmorI1[i]>Ti12[i]) stEmorI1[i]=Ti12[i]
      if (stEmord[i]>Ti12[i]) stEmord[i]=Ti12[i]
      if (stEmorI2[i]>Ti22[i]) stEmorI2[i]=Ti22[i]
      if (stEmorR1[i]>Tr12[i]) stEmorR1[i]=Tr12[i]
      if (stEmorR2[i]>Tr22[i]) stEmorR2[i]=Tr22[i]
      if (stEmigSout[i]>Ts2[i]) stEmigSout[i]=Ts2[i]
      if (stEmigI2out[i]>Ti22[i]) stEmigI2out[i]=Ti22[i]
      if (stEmigR2out[i]>Tr22[i]) stEmigR2out[i]=Tr22[i]
      if (stEmigabI[i]>Ti12[i]) stEmigabI[i]=Ti12[i]
      if (stEmigabR[i]>Tr12[i]) stEmigabR[i]=Tr12[i]
      if (stEper[i]>Te2[i]) stEper[i]=Te2[i]
    }
    
    for (i in 1:Np){
      if (i==1){
        j=Np
      } else{
        j=i-1
      }
      sk3[i]=-stEinf[i]-stEenv[i]-stEmigSout[i]-stEmorS[i]+stEbirth[i]+stEmigSout[j]
      i1k3[i]=stEinf[i]+stEenv[i]-stEmorI1[i]-stEmord[i]-stEmigabI[i]-stErec1[i]
      i2k3[i]=stEmigabI[i]-stEmigI2out[i]-stEmorI2[i]-stErec2[i]+stEmigI2out[j]
      r1k3[i]=stErec1[i]-stEmorR1[i]-stEmigabR[i]
      r2k3[i]=stErec2[i]-stEmigR2out[i]-stEmorR2[i]+stEmigR2out[j]+stEmigabR[i]
      ek3[i]=stEI[i]-stEper[i]

      Ts3[i]=S[i,cnt-1]+round(sk3[i],0)
      Ti13[i]=I1[i,cnt-1]+round(i1k3[i],0)
      Ti23[i]=I2[i,cnt-1]+round(i2k3[i],0)
      Tr13[i]=R1[i,cnt-1]+round(r1k3[i],0)
      Tr23[i]=R2[i,cnt-1]+round(r2k3[i],0)
      Te3[i]=E[i,cnt-1]+round(ek3[i],0)
    }
    
    #The forth order
    for (i in 1:Np){
      Einf[i]=tm_step*(beta[i]*Ts3[i]*(Ti13[i]+Ti23[i]))
      Eenv[i]=tm_step*(gamma*Ts3[i]*(1-exp(-alpha*Te3[i])))
      Erec1[i]=tm_step*(r*Ti13[i])
      Erec2[i]=tm_step*(r*Ti23[i])
      EmorS[i]=tm_step*(u+uh[i])*Ts3[i]
      EmorI1[i]=tm_step*(u+uh[i])*Ti13[i]
      Emord[i]=tm_step*ud*Ti13[i]
      EmorI2[i]=tm_step*(u+uh[i])*Ti23[i]
      EmorR1[i]=tm_step*(u+uh[i])*Tr13[i]
      EmorR2[i]=tm_step*(u+uh[i])*Tr23[i]
      EmigSout[i]=tm_step*m[i,k]*Ts3[i]
      EmigI2out[i]=tm_step*m[i,k]*Ti23[i]
      EmigR2out[i]=tm_step*m[i,k]*Tr23[i]
      EmigabI[i]=tm_step*v*Ti13[i]
      EmigabR[i]=tm_step*v*Tr13[i]
      Ebirth[i]=tm_step*b[i,k]
      Eper[i]=tm_step*c*Te3[i]
      EI[i]=tm_step*(Ti13[i]+Ti23[i])
    }
    
    Einf[Einf<0]=0
    Eenv[Eenv<0]=0
    Erec1[Erec1<0]=0
    Erec2[Erec2<0]=0
    EmorS[EmorS<0]=0
    EmorI1[EmorI1<0]=0
    Emord[Emord<0]=0
    EmorI2[EmorI2<0]=0
    EmorR1[EmorR1<0]=0
    EmorR2[EmorR2<0]=0
    EmigSout[EmigSout<0]=0
    EmigI2out[EmigI2out<0]=0
    EmigR2out[EmigR2out<0]=0
    EmigabI[EmigabI<0]=0
    EmigabR[EmigabR<0]=0
    Ebirth[Ebirth<0]=0
    EI[EI<0]=0
    Eper[Eper<0]=0
    
    Einf[which(is.na(Einf))]=0
    Eenv[which(is.na(Eenv))]=0
    Erec1[which(is.na(Erec1))]=0
    Erec2[which(is.na(Erec2))]=0
    EmorS[which(is.na(EmorS))]=0
    EmorI1[which(is.na(EmorI1))]=0
    Emord[which(is.na(Emord))]=0
    EmorI2[which(is.na(EmorI2))]=0
    EmorR1[which(is.na(EmorR1))]=0
    EmorR2[which(is.na(EmorR2))]=0
    EmigSout[which(is.na(EmigSout))]=0
    EmigI2out[which(is.na(EmigI2out))]=0
    EmigR2out[which(is.na(EmigR2out))]=0
    EmigabI[which(is.na(EmigabI))]=0
    EmigabR[which(is.na(EmigabR))]=0
    Ebirth[which(is.na(Ebirth))]=0
    EI[which(is.na(EI))]=0
    Eper[which(is.na(Eper))]=0
    
    #Poisson
    for (i in 1:Np){
      stEinf[i]=rpois(1,Einf[i])
      stEenv[i]=rpois(1,Eenv[i])
      stErec1[i]=rpois(1,Erec1[i])
      stErec2[i]=rpois(1,Erec2[i])
      stEmorS[i]=rpois(1,EmorS[i])
      stEmorI1[i]=rpois(1,EmorI1[i])
      stEmord[i]=rpois(1,Emord[i])
      stEmorI2[i]=rpois(1,EmorI2[i])
      stEmorR1[i]=rpois(1,EmorR1[i])
      stEmorR2[i]=rpois(1,EmorR2[i])
      stEmigSout[i]=rpois(1,EmigSout[i])
      stEmigI2out[i]=rpois(1,EmigI2out[i])
      stEmigR2out[i]=rpois(1,EmigR2out[i])
      stEmigabI[i]=rpois(1,EmigabI[i])
      stEmigabR[i]=rpois(1,EmigabR[i])
      stEbirth[i]=rpois(1,Ebirth[i])
      stEI[i]=rpois(1,EI[i])
      stEper[i]=rpois(1,Eper[i])
    }
    
    for (i in 1:Np){
      if (stEinf[i]>Ts3[i]) stEinf[i]=Ts3[i]
      if (stEenv[i]>Ts3[i]) stEenv[i]=Ts3[i]
      if (stErec1[i]>Ti13[i]) stErec1[i]=Ti13[i]
      if (stErec2[i]>Ti23[i]) stErec2[i]=Ti23[i]
      if (stEmorS[i]>Ts3[i]) stEmorS[i]=Ts3[i]
      if (stEmorI1[i]>Ti13[i]) stEmorI1[i]=Ti13[i]
      if (stEmord[i]>Ti13[i]) stEmord[i]=Ti13[i]
      if (stEmorI2[i]>Ti23[i]) stEmorI2[i]=Ti23[i]
      if (stEmorR1[i]>Tr13[i]) stEmorR1[i]=Tr13[i]
      if (stEmorR2[i]>Tr23[i]) stEmorR2[i]=Tr23[i]
      if (stEmigSout[i]>Ts3[i]) stEmigSout[i]=Ts3[i]
      if (stEmigI2out[i]>Ti23[i]) stEmigI2out[i]=Ti23[i]
      if (stEmigR2out[i]>Tr23[i]) stEmigR2out[i]=Tr23[i]
      if (stEmigabI[i]>Ti13[i]) stEmigabI[i]=Ti13[i]
      if (stEmigabR[i]>Tr13[i]) stEmigabR[i]=Tr13[i]
      if (stEper[i]>Te2[i]) stEper[i]=Te2[i]
      
    }
    
    for (i in 1:Np){
      if (i==1){
        j=Np
      } else{
        j=i-1
      }
      sk4[i]=-stEinf[i]-stEenv[i]-stEmigSout[i]-stEmorS[i]+stEbirth[i]+stEmigSout[j]
      i1k4[i]=stEinf[i]+stEenv[i]-stEmorI1[i]-stEmord[i]-stEmigabI[i]-stErec1[i]
      i2k4[i]=stEmigabI[i]-stEmigI2out[i]-stEmorI2[i]-stErec2[i]+stEmigI2out[j]
      r1k4[i]=stErec1[i]-stEmorR1[i]-stEmigabR[i]
      r2k4[i]=stErec2[i]-stEmigR2out[i]-stEmorR2[i]+stEmigR2out[j]+stEmigabR[i]
      ek4[i]=stEI[i]-stEper[i]
    }
    
    for (i in 1:Np){ 
      S[i,cnt]=S[i,cnt-1]+round(sk1[i]/6+sk2[i]/3+sk3[i]/3+sk4[i]/6)
      I1[i,cnt]=I1[i,cnt-1]+round(i1k1[i]/6+i1k2[i]/3+i1k3[i]/3+i1k4[i]/6)
      I2[i,cnt]=I2[i,cnt-1]+round(i2k1[i]/6+i2k2[i]/3+i2k3[i]/3+i2k4[i]/6)
      R1[i,cnt]=R1[i,cnt-1]+round(r1k1[i]/6+r1k2[i]/3+r1k3[i]/3+r1k4[i]/6)
      R2[i,cnt]=R2[i,cnt-1]+round(r2k1[i]/6+r2k2[i]/3+r2k3[i]/3+r2k4[i]/6)
      E[i,cnt]=E[i,cnt-1]+round(ek1[i]/6+ek2[i]/3+ek3[i]/3+ek4[i]/6)
      #newI[i,cnt]=newI[i,cnt-1]+round(ik1a[i]/6+ik2a[i]/3+ik3a[i]/3+ik4a[i]/6)
    }
    
    
  }
  
  rec=list(S=S,I=I1+I2,R=R1+R2,E); 
  rec;
  
}

#Function to calculate likelihood of virus translocation based on multiple stochastic simulations (baseline model)
SumSto<-function(yearday, year, tm_step, S0, I0, N, r, beta, u, uh, ud, b, m, Np, v, times){
  tm_vec=seq(1,year*yearday+1,by=tm_step)
  tm_sz=length(tm_vec)+1;
  S=array(0,dim=c(times,Np,tm_sz))
  I=array(0,dim=c(times,Np,tm_sz))
  R=array(0,dim=c(times,Np,tm_sz))
  avefra=vector(mode = "double",length = 27)
  for (i in 1:times){
    result=migdelayStoSIIRR(yearday, year, tm_step, S0, I0, N, r, beta, u, uh, ud, b, h, m, Np, v)
    S[i,,]=result[[1]]
    I[i,,]=result[[2]]
    R[i,,]=result[[3]]
  }
  for (i in 1:Np){
    B=0
    C=0
    for (j in 1:times){
        if ((sum(I[j,i,],na.rm = TRUE)==0)&&(sum(S[j,i,],na.rm = TRUE)!=0)){
          B=B+1
        }
        if (sum(S[j,i,],na.rm = TRUE)==0){
          C=C+1
        }
    }
    avefra[i]=1-B/times
    avefra[i+9]=B/times
    avefra[i+18]=C/times
  }
  avefra;
}

#Function to calculate likelihood of virus translocation based on multiple stochastic simulations (the secod model)
SumStoEnv<-function(yearday, year, tm_step, S0, I0, E0, N, r, beta, u, uh, ud, b, m, Np, v, c, gamma, alpha, times){
  tm_vec=seq(1,year*yearday+1,by=tm_step)
  tm_sz=length(tm_vec)+1;
  S=array(0,dim=c(times,Np,tm_sz))
  I=array(0,dim=c(times,Np,tm_sz))
  R=array(0,dim=c(times,Np,tm_sz))
  avefra=vector(mode = "double",length = 27)
  for (i in 1:times){
    result=migdelayStoSIIRREnv(yearday, year, tm_step, S0, I0, E0, N, r, beta, u, uh, ud, b, h, m, Np, v, c, gamma, alpha)
    S[i,,]=result[[1]]
    I[i,,]=result[[2]]
    R[i,,]=result[[3]]
  }
  for (i in 1:Np){
    B=0
    C=0
    for (j in 1:times){
      if ((sum(I[j,i,],na.rm = TRUE)==0)&&(sum(S[j,i,],na.rm = TRUE)!=0)){
        B=B+1
      }
      if (sum(S[j,i,],na.rm = TRUE)==0){
        C=C+1
      }
    }
    avefra[i]=1-B/times
    avefra[i+9]=B/times
    avefra[i+18]=C/times
  }
  avefra;
}
