## Script to simulate translocation of influenza in a migratory population

dir_home_code="~/Code/" # Home directory. Change accordingly

library(lhs); # for lhs
source(paste(dir_home_code,"MigDelaySIREnv.R",sep="")); # the model

tm_step=0.2;
yearday=365;
year=1;
tm_vec=seq(1,year*yearday+1,by=tm_step);
tm_sz=length(tm_vec)+1;

#define parameters
Np=9 #patch number
v=1/10
r=1/13 #recover rate
u=0.315/365 #natural mortality rate
ud=0#pathogen induced mortality rate

beta=vector(mode = "double",length = Np) #transmission rate vector
m=matrix(1,Np,yearday) #migration rate matrix
uh=vector(mode = "double",length = Np) #hunting mortality rate vector
b=matrix(0,Np,yearday) #birth rate
S0=vector(mode = "numeric",length = Np)
E0=vector(mode = "numeric",length = Np)
I0=vector(mode = "numeric",length = Np)
N=vector(mode = "numeric",length = Np)

#migration rate matrix
m[1,1:60]=0
m[1,334:365]=0
m[9,1:60]=0
m[9,334:365]=0
m[2,61:80]=0
m[3,81:100]=0
m[4,101:120]=0
m[5,121:273]=0
m[6,274:293]=0
m[7,294:313]=0
m[8,314:333]=0

#hunting martality rate
uh[1]=0.320/365
uh[6:9]=0.320/365
uh[2:5]=0

#birth rate
b[5,163:215]=40

#initral status
S0[]=0
S0[1]=4990
I0[]=0
I0[1]=10
N[]=0
N[1]=5000
E0[]=0
E0[1]=1

#Baseline model
#LHS parameters
panum=600 #number of samples
A <- randomLHS(panum, 4)
PaCom <- matrix(nrow = nrow(A), ncol = ncol(A))
#Beta
PaCom[,1] <- qunif(A[,1], max = 0.0003,min = 0.00002)
#1/r
PaCom[,2] <- qunif(A[,2], max = 13,min = 3)
#ud
PaCom[,3] <- qunif(A[,3],max = 1,min = 0)
#1/v
PaCom[,4] <- qunif(A[,4], max = 100,min = 1)

times=100
result=matrix(0,panum,31)

#run the model with each parameter combination
for (i in 1:panum){
  b0<-PaCom[i,1]
  beta[]<-b0
  beta[5]<-b0/4
  
  r1<-PaCom[i,2]
  r<-1/r1
  
  ud<-PaCom[i,3]
  
  v1<-PaCom[i,4]
  v<-1/v1
  
  result[i,1:27]=SumSto(yearday, year, tm_step, S0, I0, N, r, beta, u, uh, ud, b, m, Np, v, times)
  result[i,28:31]=c(b0,r1,ud,v1)
}


#Model with Env
#LHS parameters
panumenv=1000 #number of samples
A <- randomLHS(panumenv, 7)
PaCom <- matrix(nrow = nrow(A), ncol = ncol(A))
#Beta
PaCom[,1] <- qunif(A[,1], max = 0.0003,min = 0.00002)
#1/r
PaCom[,2] <- qunif(A[,2], max = 13,min = 3)
#ud
PaCom[,3] <- qunif(A[,3],max = 1,min = 0)
#1/v
PaCom[,4] <- qunif(A[,4], max = 100,min = 1)
#c
PaCom[,5] <- qunif(A[,5], max = 20/365,min = 2/365)
#gamma
PaCom[,6] <- qunif(A[,6], max = 0.005/365,min = 0.001/365)
#alpha
PaCom[,7] <- qunif(A[,7], max = 1000000/365,min = 1/365)

times=100
resultenv=matrix(0,panumenv,34)
for (i in 1:panumenv){
  b0<-PaCom[i,1]
  beta[]<-b0
  beta[5]<-b0/4
  
  r1<-PaCom[i,2]
  r<-1/r1
  
  ud<-PaCom[i,3]
  
  v1<-PaCom[i,4]
  v<-1/v1
  
  c<-PaCom[i,5]
  gamma<-PaCom[i,6]
  alpha<-PaCom[i,7]
  
  resultenv[i,1:27]=SumStoEnv(yearday, year, tm_step, S0, I0, E0, N, r, beta, u, uh, ud, b, m, Np, v, c, gamma, alpha, times)
  resultenv[i,28:34]=c(b0,r1,ud,v1,c,gamma,alpha)
}
